@@ -44,6 +44,14 @@
 #include "magnifier.h" /* needed to access parent data */
 #include "magnifier-private.h" /* needed to access parent data */
 
+#undef DEBUG_CLIENT_CALLS
+
+#ifdef DEBUG_CLIENT_CALLS
+#define DBG(a) a
+#else
+#define DBG(a) 
+#endif
+
 static GObjectClass *parent_class = NULL;
 
 enum {
@@ -66,6 +74,29 @@ enum {
 	ZOOM_REGION_EXIT_MAGNIFIER
 } PropIdx;
 
+#ifdef DEBUG_CLIENT_CALLS
+gchar* prop_names[ZOOM_REGION_EXIT_MAGNIFIER + 1] = 
+{
+    "MANAGED",
+    "SMOOTHSCROLL",
+    "INVERT",
+    "SMOOTHING",
+    "CONTRAST",
+    "XSCALE",
+    "YSCALE",
+    "BORDERSIZE",
+    "BORDERCOLOR",
+    "XALIGN",
+    "YALIGN",
+    "VIEWPORT",
+    "TESTPATTERN",
+    "TIMING_TEST",
+    "TIMING_OUTPUT",
+    "TIMING_PAN_RATE",
+    "EXIT_MAGNIFIER"
+};
+#endif
+
 typedef enum {
 	ZOOM_REGION_ERROR_NONE,
 	ZOOM_REGION_ERROR_NO_TARGET_DRAWABLE,
@@ -775,10 +806,9 @@ static void
 zoom_region_paint_crosswire_cursor (ZoomRegion *zoom_region, GdkRectangle *clip_rect)
 {
 	Magnifier *magnifier = zoom_region->priv->parent;
-	static GdkGC *crosswire_gc = NULL;
 	static GdkColormap *cmap;
 	static GdkColor last_color;
-	static gboolean last_color_init = !TRUE;
+	static gboolean last_color_init = FALSE;
 	GdkGCValues values;
 	GdkRectangle rect;
 	GdkDrawable *cursor;
@@ -787,14 +817,15 @@ zoom_region_paint_crosswire_cursor (ZoomRegion *zoom_region, GdkRectangle *clip_
 	int x_left_clip = 0, x_right_clip = 0, y_top_clip = 0, y_bottom_clip = 0;
 	int csize;
 	
-	if (!(magnifier && zoom_region->priv->w &&
+	if (!(magnifier && GDK_IS_DRAWABLE (zoom_region->priv->w) &&
 	      zoom_region->priv->w->window &&
 	      magnifier->crosswire_size > 0)) return;
 
-	if (crosswire_gc == NULL) 
+	if (zoom_region->priv->crosswire_gc == NULL) 
 	{
-		crosswire_gc = gdk_gc_new (zoom_region->priv->w->window);
-		cmap = gdk_gc_get_colormap(crosswire_gc);
+		zoom_region->priv->crosswire_gc = gdk_gc_new (zoom_region->priv->w->window);
+		cmap = gdk_gc_get_colormap(zoom_region->priv->crosswire_gc);
+		last_color_init = FALSE;
 	}
 
 	if (magnifier->crosswire_color == 0)
@@ -821,11 +852,11 @@ zoom_region_paint_crosswire_cursor (ZoomRegion *zoom_region, GdkRectangle *clip_
 		if (cmap)
 		{
 			gdk_rgb_find_color (cmap, &(values.foreground));
-			gdk_gc_set_values(crosswire_gc, &values, GDK_GC_FUNCTION | GDK_GC_FOREGROUND);
+			gdk_gc_set_values(zoom_region->priv->crosswire_gc, &values, GDK_GC_FUNCTION | GDK_GC_FOREGROUND);
 		}
 		else
 		{
-			gdk_gc_set_values(crosswire_gc, &values, GDK_GC_FUNCTION);
+			gdk_gc_set_values(zoom_region->priv->crosswire_gc, &values, GDK_GC_FUNCTION);
 		}
 
 		last_color.red   = color.red;
@@ -837,7 +868,7 @@ zoom_region_paint_crosswire_cursor (ZoomRegion *zoom_region, GdkRectangle *clip_
 	rect.x = zoom_region->priv->last_cursor_pos.x;
 	rect.y = zoom_region->priv->last_cursor_pos.y;
 	rect = zoom_region_view_rect_from_source_rect (zoom_region, rect);
-	if (clip_rect) gdk_gc_set_clip_rectangle (crosswire_gc, clip_rect);
+	if (clip_rect) gdk_gc_set_clip_rectangle (zoom_region->priv->crosswire_gc, clip_rect);
 
 	if (magnifier->crosswire_clip &&
 	    (cursor = magnifier_get_cursor (magnifier)))
@@ -858,31 +889,31 @@ zoom_region_paint_crosswire_cursor (ZoomRegion *zoom_region, GdkRectangle *clip_
 	{
 		if (magnifier->crosswire_clip)
 		{
-			gdk_draw_line (zoom_region->priv->w->window, crosswire_gc, rect.x, 0,
+			gdk_draw_line (zoom_region->priv->w->window, zoom_region->priv->crosswire_gc, rect.x, 0,
 				       rect.x, y_top_clip);
-			gdk_draw_line (zoom_region->priv->w->window, crosswire_gc, 0, rect.y,
+			gdk_draw_line (zoom_region->priv->w->window, zoom_region->priv->crosswire_gc, 0, rect.y,
 				       x_left_clip, rect.y);
 		}
-		gdk_draw_line (zoom_region->priv->w->window, crosswire_gc, rect.x,
+		gdk_draw_line (zoom_region->priv->w->window, zoom_region->priv->crosswire_gc, rect.x,
 			       y_bottom_clip, rect.x, 4096);
-		gdk_draw_line (zoom_region->priv->w->window, crosswire_gc, x_right_clip,
+		gdk_draw_line (zoom_region->priv->w->window, zoom_region->priv->crosswire_gc, x_right_clip,
 			       rect.y, 4096, rect.y);
 	}
 	else
 	{
 		if (magnifier->crosswire_clip )
 		{
-			gdk_draw_rectangle (zoom_region->priv->w->window, crosswire_gc, TRUE,
+			gdk_draw_rectangle (zoom_region->priv->w->window, zoom_region->priv->crosswire_gc, TRUE,
 					    rect.x - magnifier->crosswire_size / 2,
 					    0, magnifier->crosswire_size, y_top_clip);
-			gdk_draw_rectangle (zoom_region->priv->w->window, crosswire_gc, TRUE, 0,
+			gdk_draw_rectangle (zoom_region->priv->w->window, zoom_region->priv->crosswire_gc, TRUE, 0,
 					    rect.y - magnifier->crosswire_size / 2,
 					    x_left_clip, magnifier->crosswire_size);
 		}
-		gdk_draw_rectangle (zoom_region->priv->w->window, crosswire_gc, TRUE,
+		gdk_draw_rectangle (zoom_region->priv->w->window, zoom_region->priv->crosswire_gc, TRUE,
 				    rect.x - magnifier->crosswire_size / 2,
 				    y_bottom_clip, magnifier->crosswire_size, 4096);
-		gdk_draw_rectangle (zoom_region->priv->w->window, crosswire_gc, TRUE, x_right_clip,
+		gdk_draw_rectangle (zoom_region->priv->w->window, zoom_region->priv->crosswire_gc, TRUE, x_right_clip,
 				    rect.y - magnifier->crosswire_size / 2,
 				    4096, magnifier->crosswire_size);
 	}
@@ -898,7 +929,6 @@ static void
 zoom_region_paint_cursor (ZoomRegion *zoom_region,
 			  GdkRectangle *clip_rect)
 {
-	static GdkGC *paint_cursor_gc = NULL;
 	GdkGCValues values;
 	GdkRectangle rect, intersct;
 	GdkRectangle fullscreen;
@@ -917,6 +947,7 @@ zoom_region_paint_cursor (ZoomRegion *zoom_region,
 		GdkDrawable *cursor = magnifier_get_cursor (magnifier);
 		if (!cursor)
 			return;
+		else if (!GDK_IS_DRAWABLE (cursor)) g_message ("cursor isn't DRAWABLE!");
 		zoom_region->priv->cursor_backing_rect = rect;
 		if (zoom_region->priv->cursor_backing_pixels) {
 			gdk_drawable_get_size (zoom_region->priv->cursor_backing_pixels,
@@ -947,20 +978,25 @@ zoom_region_paint_cursor (ZoomRegion *zoom_region,
 				     rect.height);
 		}
 		DEBUG_RECT ("painting", rect);
-		if (cursor)
+		if (cursor && zoom_region->priv->w && GDK_IS_DRAWABLE (zoom_region->priv->w->window))
 		{
-			if (paint_cursor_gc == NULL)
-				paint_cursor_gc = gdk_gc_new (zoom_region->priv->w->window);
+		    if (zoom_region->priv->paint_cursor_gc == NULL)
+				zoom_region->priv->paint_cursor_gc = gdk_gc_new (zoom_region->priv->w->window);
 
-			gdk_gc_set_clip_rectangle (paint_cursor_gc, clip_rect);
+			gdk_gc_set_clip_rectangle (zoom_region->priv->paint_cursor_gc, clip_rect);
 			values.clip_x_origin = rect.x;
 			values.clip_y_origin = rect.y;
 			values.clip_mask = magnifier->priv->cursor_mask;
-			gdk_gc_set_values(paint_cursor_gc, &values, GDK_GC_CLIP_X_ORIGIN |
-				GDK_GC_CLIP_Y_ORIGIN | GDK_GC_CLIP_MASK);
+			gdk_gc_set_values(zoom_region->priv->paint_cursor_gc, &values, GDK_GC_CLIP_X_ORIGIN |
+					  GDK_GC_CLIP_Y_ORIGIN  | GDK_GC_CLIP_MASK);
+
+			gdk_draw_rectangle (zoom_region->priv->w->window,
+					   zoom_region->priv->paint_cursor_gc,
+					   TRUE,
+					   rect.x, rect.y, rect.width, rect.height);
 
 			gdk_draw_drawable (zoom_region->priv->w->window,
-					   paint_cursor_gc,
+					   zoom_region->priv->paint_cursor_gc,
 					   cursor,
 					   0, 0,
 					   rect.x,
@@ -1044,10 +1080,11 @@ static void
 zoom_region_paint_pixmap (ZoomRegion *zoom_region,
 			  GdkRectangle *area)
 {
+        if (!zoom_region->priv->w->window) return;
 	if (zoom_region->priv->default_gc == NULL) 
 		zoom_region->priv->default_gc = gdk_gc_new(zoom_region->priv->w->window);
 
-	if (zoom_region->priv->pixmap)
+	if (zoom_region->priv->pixmap && GDK_IS_DRAWABLE (zoom_region->priv->w->window))
 	{
 		gdk_draw_drawable (zoom_region->priv->w->window,
 				   zoom_region->priv->default_gc,
@@ -1319,6 +1356,7 @@ zoom_region_recompute_exposed_bounds (ZoomRegion *zoom_region)
 static void
 zoom_region_set_cursor_pos (ZoomRegion *zoom_region, int x, int y)
 {
+
 	if (zoom_region->priv)
 	{
 		zoom_region->priv->last_cursor_pos.x = x;
@@ -1354,67 +1392,6 @@ zoom_region_update_pointer (ZoomRegion *zoom_region, gboolean draw_cursor)
 	/* TODO: there's really no reason we should be using magnifier->priv->root here */
 	if (magnifier && magnifier->priv && magnifier_get_root (magnifier))
 	{
-#ifdef TEST_XTST_CURSOR
-		int max_cursors = XC_num_glyphs;
-		int cursor_id = 2;
-		Window window = None;
-		Display *display =
-			GDK_DISPLAY_XDISPLAY (magnifier->source_display);
-		Cursor *cursors = _get_x_cursors (display);
-		Cursor cursor = cursors [cursor_id];
-		int root_x_return, root_y_return, win_x_return, win_y_return;
-		Window root_return;
-		unsigned int mask_return;
-		GdkWindow *source_root = zoom_region_source_root (zoom_region);
-		if (source_root && display)
-		{
-			Window source_root_xid = GDK_WINDOW_XID (source_root);
-			if (source_root_xid)
-				XQueryPointer (display,
-					       source_root_xid,
-					       &root_return,
-					       &window,
-					       &root_x_return, &root_y_return,
-					       &win_x_return, &win_y_return,
-					       &mask_return);
-			fprintf (stderr, "XQueryPointer window=%d\n",
-				 (int) window);
-			if (!window) window = root_return;
-		}
-		if (window && cursors) {
-			if (!cursor_window) {
-				cursor_window = XCreateSimpleWindow (display,
-						     GDK_WINDOW_XID (source_root),
-								     0, 0,
-								     100, 100, 0,
-								     0L, 0L);
-				XMapWindow (display, cursor_window);
-			}
-			XSynchronize (display, TRUE);
-			XDefineCursor (display, cursor_window, None);
-			while (!XTestCompareCurrentCursorWithWindow (
-				       display,
-				       cursor_window) &&
-			       (cursor_id < max_cursors))
-			{
-				cursor_id += 2;
-				cursor = cursors [cursor_id];
-				XDefineCursor (
-					GDK_DISPLAY_XDISPLAY (
-						magnifier->target_display),
-					cursor_window,
-					cursor);
-				XFlush (GDK_DISPLAY_XDISPLAY (
-					       magnifier->target_display));
-				XSync (GDK_DISPLAY_XDISPLAY (
-					       magnifier->target_display),
-				       FALSE);
-			}
-			XSynchronize (display, FALSE);
-			if (cursor_id >= max_cursors) cursor_id = -1;
-			fprintf (stderr, "Current cursor id = %d\n", cursor_id);
-		}
-#endif
 		gdk_window_get_pointer (
 			magnifier_get_root (magnifier),
 			&mouse_x_return,
@@ -1476,7 +1453,7 @@ zoom_region_moveto (ZoomRegion *zoom_region,
 {
 	long dx = x * zoom_region->xscale - zoom_region->priv->exposed_bounds.x1;
 	long dy = y * zoom_region->yscale - zoom_region->priv->exposed_bounds.y1;
-/*	fprintf (stderr, "moveto %ld %ld\n", x, y); */
+/* fprintf (stderr, "moveto %ld %ld\n", x, y); */
 
 	mag_timing.dx = 0;
 	mag_timing.dy = 0;
@@ -1679,18 +1656,21 @@ zoom_region_update (ZoomRegion *zoom_region,
 			zoom_region->priv->default_gc = gdk_gc_new(zoom_region->priv->w->window);
 
 #ifndef USE_GDK_PIXBUF_RENDER_TO_DRAWABLE 
-		gdk_draw_pixbuf (zoom_region->priv->pixmap,
-				 zoom_region->priv->default_gc,
-				 zoom_region->priv->scaled_pixbuf,
-				 0,
-				 0,
-				 paint_rect.x + zoom_region->priv->exposed_bounds.x1,
-				 paint_rect.y + zoom_region->priv->exposed_bounds.y1,
-				 paint_rect.width,
-				 paint_rect.height,
-				 GDK_RGB_DITHER_NONE,
-				 0,
-				 0);
+		if (GDK_IS_DRAWABLE (zoom_region->priv->pixmap))
+		    gdk_draw_pixbuf (zoom_region->priv->pixmap,
+				     zoom_region->priv->default_gc,
+				     zoom_region->priv->scaled_pixbuf,
+				     0,
+				     0,
+				     paint_rect.x + zoom_region->priv->exposed_bounds.x1,
+				     paint_rect.y + zoom_region->priv->exposed_bounds.y1,
+				     paint_rect.width,
+				     paint_rect.height,
+				     GDK_RGB_DITHER_NONE,
+				     0,
+				     0);
+		else
+		    g_warning ("updating non-drawable pixmap");
 #else
 		gdk_pixbuf_render_to_drawable (zoom_region->priv->scaled_pixbuf,
 					       zoom_region->priv->pixmap,
@@ -2118,7 +2098,7 @@ zoom_region_set_viewport (ZoomRegion *zoom_region,
 	zoom_region->viewport = *viewport;
 #define DEBUG
 #ifdef DEBUG
-	fprintf (stderr, "setting viewport %d,%d - %d,%d\n",
+	fprintf (stderr, "Setting viewport %d,%d - %d,%d\n",
 		 (int) viewport->x1, (int) viewport->y1,
 		 (int) viewport->x2, (int) viewport->y2);
 #endif
@@ -2153,7 +2133,9 @@ zoom_region_get_property (BonoboPropertyBag *bag,
 			  gpointer user_data)
 {
 	ZoomRegion *zoom_region = user_data;
-	
+
+	DBG (fprintf (stderr, "Get zoom-region property: %s\n", prop_names[arg_id]));
+
 	switch (arg_id) {
 	case ZOOM_REGION_MANAGED_PROP:
 		BONOBO_ARG_SET_BOOLEAN (arg, zoom_region->is_managed);
@@ -2226,6 +2208,8 @@ zoom_region_set_property (BonoboPropertyBag *bag,
 	ZoomRegion *zoom_region = user_data;
 	GNOME_Magnifier_RectBounds bounds;
 
+	DBG (fprintf (stderr, "Set zoom-region property: %s\n", prop_names[arg_id]));
+
 	switch (arg_id) {
 	case ZOOM_REGION_MANAGED_PROP:
 		zoom_region->is_managed = BONOBO_ARG_GET_BOOLEAN (arg);
@@ -2265,7 +2249,6 @@ zoom_region_set_property (BonoboPropertyBag *bag,
 		break;
 	case ZOOM_REGION_BORDERSIZE_PROP:
 		zoom_region->border_size = BONOBO_ARG_GET_LONG (arg);
-	    g_message ("border size set to %d", zoom_region->border_size);
 		zoom_region_recompute_exposed_viewport (zoom_region);
 		zoom_region_update_current (zoom_region);
 		break;
@@ -2417,6 +2400,21 @@ impl_zoom_region_set_roi (PortableServer_Servant servant,
 {
 	ZoomRegion *zoom_region =
 		ZOOM_REGION (bonobo_object_from_servant (servant));
+
+	DBG (fprintf (stderr, "Set ROI: \t%d,%d %d,%d\n", 
+		      bounds->x1, bounds->y1, bounds->x2, bounds->y2));
+
+	/* if these bounds are clearly bogus, warn and ignore */
+	if (!bounds || (bounds->x2 <= bounds->x1)
+	    || (bounds->y2 < bounds->y1) || 
+	    ((bounds->x1 + bounds->x2)/2 < 0) || 
+	    ((bounds->y1 + bounds->y2)/2 < 0))
+	{
+	    g_warning ("Bad bounds request (%d,%d to %d,%d), ignoring.\n",
+		       bounds->x1, bounds->y1, bounds->x2, bounds->y2);
+	    return;
+	}
+
 	zoom_region->roi = *bounds;
 
 	if (zoom_region->timing_pan_rate > 0) {
@@ -2872,6 +2870,8 @@ zoom_region_private_init (ZoomRegionPrivate *priv)
 	priv->parent = NULL;
 	priv->w = NULL;
 	priv->default_gc = NULL;
+	priv->paint_cursor_gc = NULL;
+	priv->crosswire_gc = NULL;
 	priv->q = NULL;
 	priv->scaled_pixbuf = NULL;
 	priv->source_pixbuf_cache = NULL;
@@ -2935,6 +2935,7 @@ zoom_region_finalize (GObject *region)
 	g_object_unref (zoom_region->priv->source_pixbuf_cache);
 	g_object_unref (zoom_region->priv->scaled_pixbuf);
 	g_object_unref (zoom_region->priv->pixmap);
+	zoom_region->priv->pixmap = NULL;
 	g_object_unref (zoom_region->priv->cursor_backing_pixels);
 	g_object_unref (zoom_region->priv->border_gc);
 	g_free (zoom_region->priv);
